@@ -1,21 +1,22 @@
-#' Render server output.
+#' Renders server output.
 #'
 #' @param serverValues traditional Shiny inputs, as well as user-created values (reactive values), to be used in rendering output
+#' @param session traditional Shiny server session value
 #' @param output traditional shiny output
-#' @param serv_out_list a named list of functions that render output. Each function is of the form function(calc), where calc is a named list containing the traditional shiny input and reactive values that have calculated values derived from input. It returns the results of a shiny render function. The name of each function corresponds to its output label.
-#' @return traditional shiny output argument
-serverFunct <- function(serverValues, output, serv_out_list){
+#' @param serv_out_list a named list of functions that render output. Each function is of the form function(calc, session), where calc is a named list containing the traditional Shiny input and reactive values that have calculated values derived from input, and session is the traditional Shiny server session value. It returns the results of a Shiny render function. The name of each function corresponds to its output label.
+#' @return traditional Shiny output argument
+serverFunct <- function(serverValues, session, output, serv_out_list){
   # go through each output rendering and render
   if (length(serv_out_list) > 0){
     for (v in 1:length(serv_out_list)){
       # check for errors related to numbers of inputs
       tryCatch({
         # renderoutput and assign to output
-        output[[names(serv_out_list)[v]]] <- serv_out_list[[v]](serverValues)
+        output[[names(serv_out_list)[v]]] <- serv_out_list[[v]](serverValues, session)
       }, error = function(e){
         if (length(grep("unused argument ", as.character(e)[1], fixed = T))>0 |
             length(grep("is missing, with no default", as.character(e)[1], fixed = T))>0){
-          e$message <- paste("Argument", v, "of serv_out's functions does not have 1 argument")
+          e$message <- paste("Argument", v, "of serv_out's functions does not have 2 arguments")
         }
         stop(e)
       })
@@ -26,13 +27,18 @@ serverFunct <- function(serverValues, output, serv_out_list){
 
 #' Renders user interface for all mwshiny windows.
 #'
-#' @param win_titles vector of strings, corresponding to window titles. Must be same length as ui_win, and titles must be same index as corresponding ui page in ui_win. No windows can be named 'WindowSelector'.
+#' @param win_titles vector of uniquely named strings, corresponding to window titles. Must be same length as ui_win, and titles must be same index as corresponding ui page in ui_win. No windows can be named 'WindowSelector', and titles cannot have spaces.
 #' @param ui_list list of shiny ui pages. Must be same length as win_titles, and ui page must be same index as corresponding title in win_titles.
 #' @param depend named list of package dependencies, with the names corresponding to the packages for which .js and .css files will be imported. If value is NA, all .js and .css files will be imported. Otherwise, value must be a vector of strings corresponding to the .js and .css files to be imported, with location relative to the package folder.
 #' @return ui: user interfaces for all windows
 mwsUI <- function(win_titles, ui_list, depend) {
   win_select <- ""
   for (w in win_titles){
+    # check if titles have spaces
+    if (grepl(" ", w)){
+      stop(paste("Window titles cannot have spaces. Please remove space in window title:", w))
+    }
+
     win_select <- paste0(win_select,
                          '<h2><a href="?',w,'">',w,'</a></h2>')
   }
